#include <qlabel.h>
#include <kactivelabel.h>
#include <kconfig.h>
#include <qvgroupbox.h>
#include <qstringlist.h>

#ifndef PORTABLE_H
#define PORTABLE_H

struct power_result {
	int	powered	;	// true if we're plugged into the wall
	int	percentage;	// value 0-100 percentage of battery left
	int	time;		// time in minutes left - -1 if this is not supported by the BIOS
};


// ATTENTION: if you change something in here, please update ALL of the
//            ported sections in the implementation file!

class laptop_portable {
public:
	static void power_management_restart();	// reset internal state
	static int has_power_management();	// returns 1 if this computer has power management
	static int has_battery_time();		// returns 1 if this give BIOS battery time info
	static int has_suspend();		// returns 1 if this computer can perform a suspend
	static int has_standby();		// returns 1 if this computer can perform a standby
	static int has_hibernation();		// returns 1 if this computer can perform a hibernate
	static int has_apm(int);		// returns 1 if this computer has linux-style apm of some particular type
	static int has_apm() { return has_apm(0); }	// returns 1 if this computer has linux-style apm
	static int has_acpi(int);		// returns 1 if this computer has linux-style acpi of some particular type
	static int has_acpi() { return has_acpi(0); }	// returns 1 if this computer has linux-style acpi
	static bool has_software_suspend(int type); // // returns 1 if the system has software suspend available
	static bool has_software_suspend() { return has_software_suspend(0); }
	static int has_cpufreq();		// returns 1 if this computer can scale the cpu frequency
	static int has_brightness();		// returns 1 if this computer can set tyhe back panel brightness
	static void set_brightness(bool blank, int val);	// val = 0-255 255 brightest, 0 means dimmest (if !blank it must be still visible), 
	static int get_brightness();		// returns 0-255 brightness, -1 if you can't
	static QString cpu_frequency();		// Returns the cpu freq.
	static KActiveLabel *no_power_management_explanation(QWidget *parent);
	static QLabel *how_to_do_suspend_resume(QWidget *parent);
	static QLabel *pcmcia_info(int x, QWidget *parent);
	static void invoke_standby();
	static void invoke_suspend();
	static void invoke_hibernation();
	static struct power_result poll_battery_state();
	static void get_battery_status(int &num_batteries, QStringList &names, QStringList &state, QStringList &values); // get multiple battery status
	static bool has_lav();			// true if the following returns a valid value
	static float get_load_average();	// current short term load average
	static void extra_config(QWidget *parent, KConfig *config, QVBoxLayout *layout);
	static void acpi_set_mask(bool standby, bool suspend, bool hibernate, bool perf, bool throttle);	
	static void apm_set_mask(bool standby, bool suspend);
	static void software_suspend_set_mask(bool hibernate);
	static bool get_system_performance(bool, int&, QStringList &, bool *&);	 // do something to help get system profiles from places like ACPI
	static void set_system_performance(QString);				 // use the string returned above
	static bool get_system_throttling(bool, int&, QStringList &, bool *&);   // do something to help get system throttling data from places like ACPI
	static void set_system_throttling(QString);				 // return the string returned above
	enum LaptopButton {LidButton=0, PowerButton};
	static bool has_button(LaptopButton p);	// true if we have support for a particular button
	static bool get_button(LaptopButton p);	// true if a button is pressed
};

#endif
